/**
 * Creates and AT Internet Piano Analytics and dispatches it.
 */
(function ($, Drupal, once, drupalSettings) {

  // Process only if pianoanalytics settings have been defined.
  if (drupalSettings.pianoanalytics === undefined) {
    return;
  }

  Drupal.pianoanalytics = {

    _initialize(settings) {
      let pa_configurations = {};
      if (!!settings.site) {
        pa_configurations.site = settings.site;
      }
      if (!!settings.collect_domain) {
        pa_configurations.collectDomain = settings.collect_domain;
      }
      if (!!settings.add_event_url) {
        pa_configurations.addEventURL = settings.add_event_url;
      }
      pa.setConfigurations(pa_configurations);
    },

    _configurePrivacy(settings) {
      if (pa.privacy) {
        if (settings.cnil_exempt !== 0) {
          pa.privacy.setMode('exempt');
        }
      }
    },

    _addChaptersToProperties(settings, properties, prefix) {
      if (!!settings.page_chapter1) {
        properties[prefix + '_chapter1'] = settings.page_chapter1;
      }
      if (!!settings.page_chapter2) {
        properties[prefix + '_chapter2'] = settings.page_chapter2;
      }
      if (!!settings.page_chapter3) {
        properties[prefix + '_chapter3'] = settings.page_chapter3;
      }
    },

    _processPage(settings) {
      let page_properties = {};
      if (!!settings.page_name) {
        page_properties.page = settings.page_name;
      }
      this._addChaptersToProperties(settings, page_properties, 'page');
      if (Object.getOwnPropertyNames(page_properties).length > 0) {
        pa.sendEvent('page.display', page_properties);
      }
    },

    _triggerInitializedEvent() {
      $(document).ready(function () {
        $(document).trigger('pianoanalytics:tag_initialized');
      });
    },

    initializeAndTrackPage(settings = drupalSettings.pianoanalytics) {
      this._initialize(settings);
      this._configurePrivacy(settings);
      this._processPage(settings);
      this._triggerInitializedEvent();
    },

    /**
     * Check whether this is a download URL or not.
     *
     * @param {string} url
     *   The web url to check.
     *
     * @return {boolean} isDownload
     */
    isDownload(url) {
      var isDownload = new RegExp('\\.(' + drupalSettings.pianoanalytics.trackDownloadExtensions + ')([\?#].*)?$', 'i');
      return isDownload.test(url);
    },

    /**
     * Check whether this is an absolute internal URL or not.
     *
     * @param {string} url
     *   The web url to check.
     *
     * @return {boolean} isInternal
     */
    isInternal(url) {
      var isInternal = new RegExp('^(https?):\/\/' + window.location.host, 'i');
      return isInternal.test(url);
    },

  };

  Drupal.behaviors.pianoanalytics = {
    attach: function(context, settings) {
      once('pianoanalytics', 'html', context).forEach(function (element) {
        // Process only if pianoanalytics library have been properly loaded.
        if (window.pa === undefined) {
          return;
        }
        Drupal.pianoanalytics.initializeAndTrackPage(settings.pianoanalytics);
      });
    },
  };

  $(document).ready(function () {

    // Attach mousedown, keyup, touchstart events to document only and catch
    // clicks on all elements.
    $(document.body).on('mousedown keyup touchstart', function (event) {

      // Process only if pianoanalytics library have been properly loaded.
      if (window.pa === undefined) {
        return;
      }

      // Catch the closest surrounding link of a clicked element.
      $(event.target).closest('a,area').each(function () {

        // Is the clicked URL internal?
        if (Drupal.pianoanalytics.isInternal(this.href)) {
          // Is download tracking activated and the file extension configured
          // for download tracking?
          if (drupalSettings.pianoanalytics.trackDownload && Drupal.pianoanalytics.isDownload(this.href)) {
            // Download link clicked.
            let click_properties = {'click': this.href};
            Drupal.pianoanalytics._addChaptersToProperties(
              drupalSettings.pianoanalytics, click_properties, 'click');
            pa.sendEvent('click.download', click_properties);
          }
        }
        else {
          if (drupalSettings.pianoanalytics.trackMailto && $(this).is("a[href^='mailto:'],area[href^='mailto:']")) {
            // Mailto link clicked.
            let click_properties = {'click': this.href};
            Drupal.pianoanalytics._addChaptersToProperties(
              drupalSettings.pianoanalytics, click_properties, 'click');
            pa.sendEvent('click.action', click_properties);
          }
          else if (drupalSettings.pianoanalytics.trackOutbound && this.href.match(/^\w+:\/\//i)) {
            // External link clicked / No top-level cross domain clicked.
            let click_properties = {'click': this.href};
            Drupal.pianoanalytics._addChaptersToProperties(
              drupalSettings.pianoanalytics, click_properties, 'click');
            pa.sendEvent('click.navigation', click_properties);
          }
        }
      });
    });
  });

})(jQuery, Drupal, once, drupalSettings);
